-- GridConfigurableLayouts
-- GCLRole.lua
--
-- Created By : msaint


--{{{ Libraries and modules
local talentLib = LibStub:GetLibrary("LibGroupTalents-1.0")
local GridRoster = Grid:GetModule("GridRoster")
--}}}


--{{{ make debug function local for convenience
local function debug(...)
   GridConfigurableLayouts:Debug("[GCL-Role]", ...)
end
--}}}


---{{{ local tables and variables
local defaultConfig = {
useBlizz = true,
blizzRules = false,
}
local fixedRole = {
   ROGUE = "melee",
   HUNTER = "ranged",
   MAGE = "ranged",
   WARLOCK = "ranged",
}
local isBlizzDamageTypeAmbig = {
   SHAMAN = true,
   DRUID = true,
}
local defaultDamageRole = {
   WARRIOR = "melee",
   DEATHKNIGHT = "melee",
   PALADIN = "melee",
}
setmetatable(defaultDamageRole,{ __index = function(l, class) return fixedRole[class] or "ranged" end})
-- I only include fixedRole in meta table return for completeness: as written it will never be used because fixedRole checking always precedes defaultDamageRole use
local Role = {}
local guidsToUpdate = {}
---}}}


---{{{ local functions
local function getTalentRole(guid, unit, class, forceRefresh)
   local role = talentLib:GetGUIDRole(guid, forceRefresh)
   role = role and role:gsub("caster", "ranged") -- the caller must deal with the strange treatment of hunters (preferably by using fixedRole[class] to override)
   debug("Talentlib role for", unit, "is", role) 
   return role or nil
end


local function getBlizzRole(unit, class)
   local role = UnitGroupRolesAssigned(unit)
   role = role and role:lower()
   role = (role ~= "none"  and role ~= "damager" and role) or (role == "damager" and not isBlizzDamageTypeAmbig[class] and defaultDamageRole[class]) or nil
   debug("Blizzrole for", unit, "is", role)
   return role
end
---}}}


--{{{ methods
function GridConfigurableLayouts:GetRole(guid, unit, class, forceRefresh)   -- only first is mandatory
   unit = unit or GridRoster:GetUnitidByGUID(guid)
   if not class then
      _, class = UnitClass(unit)
   end
   useBlizz = GridConfigurableLayouts.db.profile.useBlizzRoles
   blizzRules = GridConfigurableLayouts.db.profile.blizzRolesRule
   local role = Role[guid]
   debug("GetRole is processing", unit, "with existing role", role)
   if not role or forceRefresh then
      role = useBlizz and getBlizzRole(unit, class)
      role = fixedRole[class] or (blizzRules and role) or getTalentRole(guid, unit, class, forceRefresh) or role or defaultDamageRole[class]
      Role[guid] = role
      debug("GetRole has set", unit, "to", role)
   end
   return role
end


function GridConfigurableLayouts:wipeRoles()
   Role = Role and wipe(Role)
end


function GridConfigurableLayouts:RegisterRoleEvents()
   self:RegisterEvent("ROLE_CHANGED_INFORM") --, GridConfigurableLayouts.ROLE_CHANGED_INFORM)
   talentLib.RegisterCallback(GridConfigurableLayouts, 'LibGroupTalents_RoleChange')
end


function GridConfigurableLayouts:UnregisterRoleEvents()
   self:UnregisterMessage("ROLE_CHANGED_INFORM")
   talentLib.UnregisterCallback(GridConfigurableLayouts, 'LibGroupTalents_RoleChange')
end


function GridConfigurableLayouts:LibGroupTalents_RoleChange(event, guid, unit, newrole)
   local _, class = UnitClass(unit)
   debug("Talent rolechange event for", unit)
   oldrole = Role[guid]
   newrole = GridConfigurableLayouts:GetRole(guid, unit, class, true)
   if oldrole ~= newrole then
      GridConfigurableLayouts:UpdateGroups()
   end
end


function GridConfigurableLayouts:ROLE_CHANGED_INFORM(event, name, oldname, oldrole, role)
   name = name and name:gsub("(%a)(%a+)", function (a,b) return a:upper() .. b:lower() end)
   oldname = oldname and oldname:gsub("(%a)(%a+)", function (a,b) return a:upper() .. b:lower() end)
   debug("Blizzard rolechange event for", name, ": new unconverted blizzrole is", role)
   -- if we check blizzrole in this thread, it hasn't changed yet, so we need to schedule an update (and, from experience, it can't be very soon)
   if name then
      local guid = GridRoster:GetGUIDByFullName(name)
      if guid then
         table.insert(guidsToUpdate, guid)
      end
   end
   if oldname and oldname ~= name then
      local guid = GridRoster:GetGUIDByFullName(oldname)
      if guid then
         table.insert(guidsToUpdate, guid)
      end
   end
   if guidsToUpdate then
      self:CancelTimer(GridConfigurableLayouts.roleUpdateTimer, true)
      GridConfigurableLayouts.roleUpdateTimer = self:ScheduleTimer(GridConfigurableLayouts.updateRoles, 1)      
   end
end


function GridConfigurableLayouts:updateRoles()
   if guidsToUpdate then
      local updateneeded
      for i = table.getn(guidsToUpdate), 1, -1 do
         local guid = guidsToUpdate[i]
         table.remove(guidsToUpdate, i)
         if GridConfigurableLayouts.debugging then
            debug("Updating roles from blizz rolechange events for", GridRoster:GetUnitidByGUID(guid))
         end                               
         oldrole = Role[guid]
         newrole = GridConfigurableLayouts:GetRole(guid, nil, nil, true)   
         updateneeded = updateneeded or oldrole ~= newrole
      end
      if updateneeded then
         GridConfigurableLayouts:UpdateGroups()
      end
   end
end
--}}}
